import { RabbitMQModule } from '@golevelup/nestjs-rabbitmq';
import { Module } from '@nestjs/common';
import { ConfigModule, ConfigService } from '@nestjs/config';
import { AdMessagerController } from './adapters/primaries/ad-messager.controller';
import { AdProfile } from './mappers/ad.profile';
import { CreateAdUseCase } from './domain/usecases/create-ad.usecase';
import { AdRepository } from './adapters/secondaries/ad.repository';
import { DatabaseModule } from '../database/database.module';
import { CqrsModule } from '@nestjs/cqrs';
import { Messager } from './adapters/secondaries/messager';
import { GeoTimezoneFinder } from '../geography/adapters/secondaries/geo-timezone-finder';
import { DefaultParamsProvider } from './adapters/secondaries/default-params.provider';
import { GeorouterCreator } from '../geography/adapters/secondaries/georouter-creator';
import { GeographyModule } from '../geography/geography.module';
import { HttpModule } from '@nestjs/axios';
import { PostgresDirectionEncoder } from '../geography/adapters/secondaries/postgres-direction-encoder';

@Module({
  imports: [
    GeographyModule,
    DatabaseModule,
    CqrsModule,
    HttpModule,
    RabbitMQModule.forRootAsync(RabbitMQModule, {
      imports: [ConfigModule],
      useFactory: async (configService: ConfigService) => ({
        exchanges: [
          {
            name: configService.get<string>('RMQ_EXCHANGE'),
            type: 'topic',
          },
        ],
        handlers: {
          adCreated: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'ad.created',
            queue: 'matcher-ad-created',
          },
        },
        uri: configService.get<string>('RMQ_URI'),
        connectionInitOptions: { wait: false },
        enableControllerDiscovery: true,
      }),
      inject: [ConfigService],
    }),
  ],
  controllers: [AdMessagerController],
  providers: [
    {
      provide: 'ParamsProvider',
      useClass: DefaultParamsProvider,
    },
    {
      provide: 'GeorouterCreator',
      useClass: GeorouterCreator,
    },
    {
      provide: 'TimezoneFinder',
      useClass: GeoTimezoneFinder,
    },
    {
      provide: 'DirectionEncoder',
      useClass: PostgresDirectionEncoder,
    },
    AdProfile,
    Messager,
    AdRepository,
    CreateAdUseCase,
  ],
  exports: [],
})
export class AdModule {}
